<?php declare(strict_types=1);

namespace App\Enums;

use Illuminate\Support\Collection;

enum AccessTokenPermission: string
{
    case RepoReadWriteDelete = 'repo:read.write.delete';
    case RepoReadWrite       = 'repo:read.write';
    case RepoRead            = 'repo:read';

    public function getTitle(): string
    {
        return __('access-token-permission.' . $this->name);
    }

    public static function toArray(): array
    {
        $choices = [];
        foreach (self::cases() as $lang) {
            $choices[] = [
                'name' => $lang->name,
                'value' => $lang->value,
                'title' => $lang->getTitle(),
            ];
        }
        return $choices;
    }

    public static function toCollection(): Collection
    {
        return collect(self::toArray());
    }
}
