<?php declare(strict_types=1);

namespace App\Http\Controllers\Registry;

use App\Http\Requests\Registry\Version2\AuthRequest;
use App\Http\Requests\Registry\Version2\EventRequest;
use App\Services\Registry\V2\AuthorizationService;
use App\Services\Registry\V2\EventService;
use Illuminate\Http\JsonResponse;
use Illuminate\Support\Facades\Log;

final class Version2 extends Controller
{
    public function __construct(
        private readonly AuthorizationService $authorizationService,
        private readonly EventService         $eventService,
    ) { }

    public function auth(AuthRequest $request): JsonResponse
    {
        $data = $request->getDto();
        $result = $this->authorizationService->authorization($data);
        if (!$result->isSuccess()) {
            return response()->json($result->getData())->setStatusCode($result->getCode());
        }

        return \response()->json($result->getData());
    }

    public function event(EventRequest $request): JsonResponse
    {
        $data = $request->getDto();
        $result = $this->eventService->dataProcessing($data);
        if (!$result->isSuccess()) {
            return response()->json($result->getData())->setStatusCode($result->getCode());
        }

        return \response()->json(['message' => $result->getMessage()]);
    }
}
