<?php declare(strict_types=1);

namespace App\Services\TagRepository;

use App\Models\Repository;
use App\Models\TagRepository;
use App\ServiceResults\ServiceResultError;
use App\ServiceResults\TagRepository\IncrementPullResult;
use App\Services\Service;

final class IncrementPullCommand extends Service
{
    public function execute(Repository $repository, string $tagName, string $digest): ServiceResultError | IncrementPullResult
    {
        $tag = $repository->tags()
                ->withTrashed()
                ->where('name', $tagName)
                ->where('digest', $digest)
                ->first();

        if (null === $tag) {
            return $this->errNotFound(__('Not Found') . ' Tag: ' . $tagName . ':' . $digest);
        }

        Repository::withTrashed()->where('id', $repository->id)
            // So that the updated_at field is not updated
            ->where(fn($q) => $q->getModel()->timestamps = false)
            ->increment('quantity_pulls');

        TagRepository::withTrashed()->where('id', $tag->id)
            // So that the updated_at field is not updated
            ->where(fn($q) => $q->getModel()->timestamps = false)
            ->increment('quantity_pulls');

        return new IncrementPullResult($repository, $tag);
    }
}
