<?php declare(strict_types=1);

namespace App\Services\TagRepository;

use App\Models\Architecture;
use App\Models\Repository;
use App\Models\TagRepository;

final readonly class UpdateOrCreateCommand
{
    public function __construct(
        private TagRepositoryCommandHandler $tagRepositoryCommandHandler,
    ) { }

    public function execute(Repository $repository, Architecture $architecture, string $name, array $data): TagRepository
    {
        $tag = TagRepository::withTrashed()
            ->where('repository_id', $repository->id)
            ->where('architecture_id', $architecture->id)
            ->where('name', $name)
            ->first();

        if (null === $tag) {
            return $this->tagRepositoryCommandHandler->handleStore($repository, $architecture, $name, $data);
        }

        return $this->tagRepositoryCommandHandler->handleUpdate($tag, $data);
    }
}
